﻿# usage -xmlconfig C:\osc\new.xml -applicationsxml C:\osc\applications.xml
param ([string] $hypervisorconfig, [string] $clonevm)

# add-xenserver snapin
$snapins  = Get-PSSnapin XenServerPSSnapIn -ea 0
if ($snapins -eq $null) {
   Get-PSSnapin -Registered "XenServerPSSnapIn" | Add-PSSnapin
}

function restart_osc_vm  {

	Write-Output "restarting $vm_fqdn in 5 seconds"
	connect-xenserver -url $xenserver_url -UserName $xenserver_user -Password $xenserver_password | Out-Null
#	Start-Sleep 5
	Invoke-XenServer:VM.CleanReboot -VM $vm_name
}


function xenserver_create_vm  {

	try {
		connect-xenserver -url $xenserver_url -UserName $xenserver_user -Password $xenserver_password | Out-Null
	} catch {
		Write-Output "connection to $xenserver_url not possible"
		exit -1
	}

	# not using Get-XenServer:VM -NameFilter $vm_name not working because of ugly error message
	$vms = Get-XenServer:VM 
	foreach ($vm in $vms) {
		if ($vm.name_label -ieq $vm_name) {
			Write-Output "VM $vm_name already found on XenServer aborting"
			exit -1	
		}
	}	 

	# copy vm from template
	$strtmp = "creating VM "  + $vm_name + " from template " + $vm_template 
	Write-Output $strtmp
	Invoke-XenServer:VM.Clone -VM $vm_template -NewName $vm_name | Out-Null
	invoke-xenserver:VM.Provision -VM $vm_name | Out-Null

	# set the mac if nessary xenserver_create_vm
	if ($vm_mac -ne "") {
		$vm=Get-XenServer:VM -NameFilter $vm_name
		# Destroy interface 0
		Destroy-XenServer:VIF -VIF $VM.VIFs[0]
		# Get selected network
		$vm_network_xe =Get-XenServer:Network -NameFilter $vm_network
		# Create VIF and attach to network
		Create-XenServer:VIF -VM $vm.uuid -Network $vm_network_xe.uuid -Device 0 -MAC "$vm_mac" |Out-Null
		Write-output "set $vm_mac at $vm_network"
	}
	
	Invoke-XenServer:VM.Start -VM $vm_name | Out-Null
	Write-Output "VM $vm_name started"
}


function Test-TcpPort ([string] $targetname, [int] $port) {

#Write-Output "testing..."
$socket = new-object Net.Sockets.TcpClient
$socket.ReceiveTimeout = 200
$socket.SendTimeout = 200

	try {
		$socket.Connect($targetname, $port)
		if ($socket.Connected) {
		$status = "Open"
		$statusreturn = $true
		$socket.Close()
		}
		else {
		$status = "Closed / Filtered"
		$statusreturn = $false
		}
	} catch { 
		$status = "Closed / Filtered" 
		$statusreturn = $false
		}
	
	$socket = $null
	#Write-Host "$targetname`t$port`t$status"
	return $status
}

function wait_for_vm {

	# wait for VM to be started

	$vm_started = 0
	$n = 0
	$retry_sec = 5
	[int] $n_max = $vm_startuptimeout / $retry_sec

	connect-xenserver -url $xenserver_url -UserName $xenserver_user -Password $xenserver_password | Out-Null

	Write-Output "wait for $vm_name to come up" 

	$ip = ""
	do {
		$n++
		
		$test = Get-XenServer:VM.GuestMetrics -VM $vm_name 
		$ip = $test.networks.Values
		if ($ip -match "\d{1,3}.\d{1,3}.\d{1,3}.\d{1,3}") {
			Write-Output "determined $ip for $vm_name"
			$vm_started = 1
			
			# ip determined by xenserver tools now checking connectivity again
			# testing ip connectivity TCP/445 should be fine
			$tcpresult = Test-TcpPort $ip $vm_testport
			Write-Output "tcpresult: $tcpresult"
			if ($tcpresult -ieq "open") {
				Write-Output "connect to $ip on port $vm_testport succeeded" 
				$vm_started=1
			} else {
				$vm_startup_left = $vm_startuptimeout - ($n * $retry_sec)
				write-output "waiting for VM $vm_name ... $n run - waiting left $vm_startup_left secs" 
				# jump out after some time
				if (($n * $retry_sec) -ge $vm_startuptimeout) {
					write-error "$vm_name did not respond within $vm_startuptimeout seconds. quitting .. "
					exit -1
				}
				$vm_started=0
				Start-Sleep $retry_sec
			}

		} else {
			$vm_startup_left = $vm_startuptimeout - ($n * $retry_sec)
			write-output "waiting for VM $vm_name ... $n run - waiting left $vm_startup_left secs" 
			if (($n * $retry_sec) -ge $vm_startuptimeout) {
				write-error "$vm_name did not respond within $vm_startuptimeout seconds. quitting .. "
				exit -1
			}
			
			Start-Sleep $retry_sec
		}


	} while ($vm_started -ne 1)


}

function prepair_osc_guest {
	Write-output "waiting some more secs to be sure server is up"
	connect-xenserver -url $xenserver_url -UserName $xenserver_user -Password $xenserver_password | Out-Null
	
	# determine the vm IP
	$test = Get-XenServer:VM.GuestMetrics -VM $vm_name 
	$ip = $test.networks.Values
	
	net use \\$ip\c$ /user:$credentials_username $credentials_password
	Write-Output "net use done"
	
	$vm_cifs = "\\$ip\c$\sysprep"
	if (-not (Test-Path $vm_cifs )) {
		New-Item -Path $vm_cifs -type directory |Out-Null
	}

	$content = Get-Content $vm_syspreptemplate

	for($i = 0;$i-lt$content.Length;$i++) {
		$content[$i] = $content[$i]  -replace "REPLACECOMPUTERNAME",$vm_name
	}

	$out_file = $vm_cifs + "\sysprep.xml"
	# create sysprep.xml
	set-content -Path $out_file -Value $content  -Force
	
	$strtmp = $vm_cifs+"\startsysprep.cmd"
	$strtmp1 = $script_workingdir+"\startsysprep.cmd"
	Copy-Item $strtmp1 $strtmp

	# only without domain
	$credentials_domain = "."
	
	$seq_winlogon = "\\"+$ip+"\HKLM\Software\Microsoft\Windows NT\CurrentVersion\Winlogon"
	

	$reg_path = $script_workingdir+'\reg.exe'
	if (-not (test-path $reg_path)) {
		Write-Error "$reg_path not found - please make sure it exists on targetpath"
	}
	
	Start-Process $reg_path -ArgumentList "delete ""$seq_winlogon"" /v AutoLogonCount  /f "

	Start-Process $reg_path -ArgumentList "add ""$seq_winlogon"" /v DefaultUserName /t REG_SZ /d $credentials_username /f " -Wait
	Start-Process $reg_path -ArgumentList "add ""$seq_winlogon"" /v DefaultPassword /t REG_SZ /d $credentials_password /f " -Wait
	Start-Process $reg_path -ArgumentList "add ""$seq_winlogon"" /v DefaultDomainName /t REG_SZ /d $credentials_domain /f " -Wait
	Start-Process $reg_path -ArgumentList "add ""$seq_winlogon"" /v AutoAdminLogon /t REG_SZ /d 1 /f " -Wait

	$strtmp = "\\"+$ip+"\c$\ProgramData\Microsoft\Windows\Start Menu\Programs\Startup\startseq.cmd.lnk"
	$strtmp1 = $script_workingdir + "\startsysprep.cmd.lnk" 
	Copy-Item -Path $strtmp1 -Destination $strtmp -force

	Write-Output "startuplink copied to $strtmp"
	net use \\$ip\c$ /d
}

# check prerequisits
$script_workingdir = Split-Path -Parent $MyInvocation.MyCommand.Path
$strtmp = $script_workingdir + '\reg.exe'
if (-not (test-path $strtmp)) {
	Write-Error "$strtmp not found - please make sure it exists on targetpath"
}

# check if xml files are present
if ( -not (Test-Path $hypervisorconfig)) {
	Write-Error $hypervisorconfig " file not found! check -hypervisorconfig parameter "
	exit -1
} elseif ( -not (Test-Path $clonevm)) {
	Write-Error $applicationsxml " file not found! check -clonevm config parameter"
	exit -1
}

	
$ip = ""

[xml] $configfile = Get-Content $hypervisorconfig
$hypervisor = $configfile.OSCustomizing.Hypervisor

$xenserver_url 	= $hypervisor.url
$xenserver_fqdn	= $hypervisor.fqdn
$xenserver_user = $hypervisor.user
$xenserver_password = $hypervisor.password

[xml] $configfile = Get-Content $clonevm
$vm_name = $configfile.OSCustomizing.VM.name
$vm_template = $configfile.OSCustomizing.VM.template
$vm_testport = $configfile.OSCustomizing.VM.testport
$vm_startuptimeout = $configfile.OSCustomizing.VM.startuptimeout
$vm_syspreptemplate = $configfile.OSCustomizing.VM.syspreptemplate
$vm_mac = $configfile.OSCustomizing.VM.mac
$vm_network = $configfile.OSCustomizing.VM.network


if (-not (Test-Path $vm_syspreptemplate)) {
	Write-Output "sysprep file $vm_syspreptemplate not found"
	exit -1
}

$credentials_domain = $configfile.OSCustomizing.VM.domain
$credentials_username = $configfile.OSCustomizing.VM.username
$credentials_password = $configfile.OSCustomizing.VM.password

xenserver_create_vm
wait_for_vm
prepair_osc_guest
restart_osc_vm

Write-Output "os customizing for $vm_name done"

